<?php
/**
 *
 *   Copyright © 2010-2023 by xhost.ch GmbH
 *
 *   All rights reserved.
 *
 **/

class FtpClientController extends Controller
{
    public $layout='//layouts/column2';

    public $statusInit = false;
    public $statusMsg = '';
    public $statusProgress = 0;
    public $statusCount = 0;
    public $statusStep = 20;

    public function filters()
    {
        return array(
            'accessControl',
        );
    }

    public function accessRules()
    {
        return array(
            array((Yii::app()->params['ftp_client_disabled'] === true) ? 'deny' : 'allow',
                'actions'=>array('index', 'login', 'createAccount'),
                'users'=>array('@'),
            ),
            array((Yii::app()->params['ftp_client_type'] != 'net2ftp') ? 'allow' : 'deny',
                'actions'=>array('files', 'tree', 'download', 'move', 'delete', 'copy', 'rename', 'newDirectory', 'newFile', 'upload', 'zip', 'unzip', 'view', 'edit'),
                'users'=>array('@'),
            ),
            array((Yii::app()->params['ftp_client_type'] != 'net2ftp') ? 'deny' : 'allow',
                'actions'=>array('browse'),
                'users'=>array('@'),
            ),
            array('deny',
                'users'=>array('*'),
            ),
        );
    }

    public function actionIndex($id = 0)
    {
        $this->redirect(array('ftpClient/login', 'id'=>(int)$id));
    }

    public function getFtpServer($server)
    {
        $daemon = Daemon::model()->findByPk((int)$server->daemon_id);
        if (!$daemon)
            throw new CHttpException(404, Yii::t('mc', 'No daemon found for this server.'));
        if (!isset($daemon->ftp_ip) || !isset($daemon->ftp_port))
            throw new CHttpException(500, Yii::t('mc', 'Daemon database not up to date, please run the Multicraft installer.'));
        return array('ip'=>$daemon->ftp_ip, 'port'=>$daemon->ftp_port);
    }

    public function getUsername($server)
    {
        return Yii::app()->user->name.'.'.$server->id;
    }

    public function net2FtpDefines()
    {
        define("NET2FTP_APPLICATION_ROOTDIR", dirname(__FILE__).'/../extensions/net2ftp/');
        if     (isset($_SERVER["SCRIPT_NAME"]) == true) { define("NET2FTP_APPLICATION_ROOTDIR_URL", dirname($_SERVER["SCRIPT_NAME"])); }
        elseif (isset($_SERVER["PHP_SELF"]) == true)    { define("NET2FTP_APPLICATION_ROOTDIR_URL", dirname($_SERVER["PHP_SELF"])); }
    }

    public function actionLogin($id = 0)
    {
        $server = Server::model()->findByPk((int)$id);
        if (!$server)
            throw new CHttpException(404, Yii::t('mc', 'The requested page does not exist.'));
        if (isset($_POST['password']))
        {
            $pw = $_POST['password'];
            Yii::log('Logging in to FTP server for server '.$server->id);
            if (@Yii::app()->params['ftp_client_type'] != 'net2ftp')
            {
                if (strlen((string)$_POST['password']))
                    $_SESSION['ftp_pw'] = $pw;

                return $this->redirect(array('ftpClient/files', 'id'=>$server->id));
            }
            $this->net2FtpDefines();
            global $net2ftp_result, $net2ftp_settings, $net2ftp_globals;
            require_once(dirname(__FILE__).'/../extensions/net2ftp/includes/main.inc.php');
            require_once(dirname(__FILE__).'/../extensions/net2ftp/includes/authorizations.inc.php');
            $ftpSv = $this->getFtpServer($server);
            if (strlen((string)$pw))
            {
                $_SESSION['net2ftp_password_encrypted'] = encryptPassword($pw);
                $sessKey = 'net2ftp_password_encrypted_'.$ftpSv['ip'].$this->getUsername($server);
                unset($_SESSION[$sessKey]);
            }
            $this->redirect(array('ftpClient/browse', 'id'=>$server->id));
        }

        $daemon = Daemon::model()->findByPk($server->daemon_id);
        $access = Yii::app()->user->model->getServerFtpAccess($server->id);
        $permissions = ($access == 'rw' ? 'elradfmw' : ($access == 'ro' ? 'elr' : ''));

        $this->render('login',array(
            'server'=>$server,
            'havePw'=>isset($_SESSION[(@Yii::app()->params['ftp_client_type'] != 'net2ftp') ? 'ftp_pw' : 'net2ftp_password_encrypted']),
            'daemon'=>$daemon,
            'permissions'=>$permissions,
        ));
    }

    public function actionCreateAccount($id)
    {
        if(!Yii::app()->request->isPostRequest)
            Yii::app()->user->deny();
        Yii::app()->user->can($id, 'manage_users', true);
        $server = Server::model()->findByPk((int)$id);
        if (!$server)
            throw new CHttpException(404, Yii::t('mc', 'The requested page does not exist.'));
        $cfg = ServerConfig::model()->findByPk($server->id);
        if ($server->id && (Yii::app()->user->isStaff() || (Yii::app()->user->can($id, 'manage_ftp') && $cfg->user_ftp)))
            Yii::app()->user->model->setServerFtpAccess($server->id, 'rw');
        else
            Yii::app()->user->deny();
        $this->redirect(array('ftpClient/login', 'id'=>$server->id));
    }

    public function path($base, $name)
    {
        return preg_replace(',/+,', '/', $base.'/'.$name);
    }

    public function up($path)
    {
        $up = preg_replace(',/([^/]|/+$)*$,', '', (string)$path);
        return strlen((string)$up) ? $up : '/';
    }

    public function editLimit()
    {
        $max = $this->bytesFromString(@ini_get('post_max_size'));
        $limit = (int)@Yii::app()->params['ftp_client_max_edit'] * 1024 * 1024;
        if (!$limit || $limit > $max)
            $limit = $max;
        return $limit;
    }

    public function canEdit($size)
    {
        $limit = $this->editLimit();
        return !$limit || ($limit >= $size);
    }

    public function uploadLimit()
    {
        $max = $this->bytesFromString(@ini_get('upload_max_filesize'));
        $limit = (int)@Yii::app()->params['ftp_client_max_upload'] * 1024 * 1024;
        if (!$limit || $limit > $max)
            $limit = $max;
        return $limit;
    }

    public function canUpload($size)
    {
        $limit = $this->uploadLimit();
        return !$limit || ($limit >= $size);
    }

    public function downloadLimit()
    {
        return (int)@Yii::app()->params['ftp_client_max_download'] * 1024 * 1024;
    }

    public function canDownload($size)
    {
        $limit = $this->downloadLimit();
        return !$limit || ($limit >= $size);
    }

    public function js($str)
    {
        if (is_array($str))
            return '""';
        return '"'.CJavaScript::quote((string)$str).'"';
    }

    public function actionNewDirectory($id, $path)
    {
        if(!Yii::app()->request->isPostRequest)
            Yii::app()->user->deny();

        list($server, $ftp) = $this->loginFtp($id);

        $name = $_POST['name'];;

        if (!@ftp_mkdir($ftp, $this->path($path, $name)))
            Yii::app()->end(Yii::t('mc', 'Failed to create directory "{name}"', array('{name}'=>$name)));
    }

    public function actionNewFile($id, $path)
    {
        if(!Yii::app()->request->isPostRequest)
            Yii::app()->user->deny();

        list($server, $ftp) = $this->loginFtp($id);

        $name = $_POST['name'];;

        $fp = fopen('php://temp', 'r+');

        $msg = '';
        if (!ftp_fput($ftp, $this->path($path, $name), $fp))
            $msg = Yii::t('mc', 'Failed to create file "{name}"', array('{name}'=>$name));

        fclose($fp);

        Yii::app()->end($msg);
    }

    public function listFiles($ftp, $path, $recursive = false)
    {
        $list = ftp_rawlist($ftp, $path);
        if ($list === false)
            return false;

        $files = array();
        foreach ($list as $item)
        {
            $split = preg_split('/\s+/', $item, 8);
            list($time, $name) = explode(' ', (string)@$split[7], 2);
            if (in_array($name, array('', '.', '..')))
                continue;
            $inYear = strpos($time, ':');
            $type = @$split[0][0] == 'd' ? 'directory' : 'file';
            $files[$this->path($path, $name)] = array(
                'type'=>$type,
                'size'=>(int)@$split[4],
                'time'=>strtotime(@$split[5].' '.@$split[6].' '.($inYear ? $time.' GMT' : $time)),
                'inYear'=>$inYear,
                'name'=>$name,
            );
            if ($recursive && $type == 'directory')
            {
                $sub = $this->listFiles($ftp, $this->path($path, $name), $recursive);
                if ($sub)
                {
                    foreach ($sub as $p=>$s)
                        $files[$p] = $s;
                }
            }
        }
        return $files;
    }

    public function actionFiles($id, $path = '', $inline = false)
    {
        $path = '/'.join('/', array_filter(explode('/', $path), 'strlen'));

        $server = Server::model()->findByPk((int)$id);
        if (!$server)
            throw new CHttpException(404, Yii::t('mc', 'The requested page does not exist.'));

        list($server, $ftp) = $this->loginFtp($server, !$inline);

        $files = $this->listFiles($ftp, $path);
        if ($files === false)
            throw new CHttpException(500, Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'listFiles')));

        $params = array(
            'server'=>$server,
            'ftp'=>$ftp,
            'files'=>$files,
            'path'=>$path,
        );
        if ($inline)
            $this->renderPartial('filesList', $params);
        else
            $this->render('files', $params);
    }

    public function actionTree($id, $path = '')
    {
        list($server, $ftp) = $this->loginFtp($id);

        $files = $this->listFiles($ftp, $path);
        if ($files === false)
            throw new CHttpException(500, Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'listFiles')));

        foreach ($files as $k=>$v)
        {
            if ($v['type'] != 'directory')
                unset($files[$k]);
        }

        $this->renderPartial('tree', array(
            'server'=>$server,
            'ftp'=>$ftp,
            'files'=>$files,
            'path'=>$path,
        ));
    }

    public function status($msg)
    {
        if (!$this->statusInit)
        {
            @ob_end_flush();
            @ob_implicit_flush(true);

            echo str_pad(' ', 1024)."\n";
            $this->statusInit = true;
        }
        echo $msg."\n";
    }

    public function actionUpload($id, $path)
    {
        if(!Yii::app()->request->isPostRequest)
            Yii::app()->user->deny();

        list($server, $ftp) = $this->loginFtp($id);

        $target = @$_POST['target'];
        if (!$target)
            $target = $path;

        $files = @$_FILES['file_upload'];
        if (!$files)
            Yii::app()->end(Yii::t('mc', 'No files uploaded'));
        foreach ($files['name'] as $i=>$name)
        {
            if (!$this->canUpload($files['size'][$i]))
                Yii::app()->end(Yii::t('mc', 'File "{name}" is too big', array('{name}'=>$name)));
            $fp = fopen($files['tmp_name'][$i], 'r');
            if (!ftp_fput($ftp, $this->path($target, $name), $fp, FTP_BINARY))
                Yii::app()->end(Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'ftp_fput')));
            fclose($fp);
        }
    }

    public function actionDownload($id, $path)
    {
        list($server, $ftp) = $this->loginFtp($id);

        $size = ftp_size($ftp, $path);
        if ($size === false)
            throw new CHttpException(500, Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'ftp_size')));

        if (!$this->canDownload($size))
            throw new CHttpException(400, Yii::t('mc', 'File "{name}" is too big', array('{name}'=>$path)));

        header('Content-Type: application/octet-stream');
        header('Content-Disposition: attachment; filename='.basename($path));
        header('Content-Length: '.$size); 

        if (!ftp_get($ftp, 'php://output', $path, FTP_BINARY))
            throw new CHttpException(500, Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'ftp_get')));
    }

    public function statusProgressInit($msg, $count)
    {
        $this->status($msg);
        $this->statusMsg = $msg;
        $this->statusProgress = 0;
        $this->statusCount = $count;
    }

    public function statusProgress()
    {
        if (!(++$this->statusProgress % $this->statusStep))
            $this->status($this->statusMsg.' ('.$this->statusProgress.'/'.$this->statusCount.')');
    }

    public function actionMove($id, $path)
    {
        if(!Yii::app()->request->isPostRequest)
            Yii::app()->user->deny();

        list($server, $ftp) = $this->loginFtp($id);

        $target = $_POST['target'];
        $items = $_POST['items'];

        $fails = array();
        $this->statusProgressInit(Yii::t('mc', 'Moving...'), count($items));
        foreach ($items as $item)
        {
            if (!ftp_rename($ftp, $this->path($path, $item), $this->path($target, $item)))
                $fails[] = $item;
            $this->statusProgress();
        }
        $this->status(Yii::t('mc', 'Done'));

        $msg = 'SUCCESS';
        if (count($fails))
        {
            $msg = Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'ftp_rename')).':';
            $msg .= "\n".join("\n", $fails);
        }

        Yii::app()->end($msg);
    }

    public function daemonDelete($id, $path, $files, $dirs)
    {
        $server = Server::model()->findByPk((int)$id);
        if (!$server)
            throw new CHttpException(404, Yii::t('mc', 'The requested page does not exist.'));

        $items = array_merge($files, $dirs);

        $cmd = array();
        foreach ($items as $item)
            $cmd[] = $this->relative($this->path($path, $item));
        if (!McBridge::get()->serverCmd($server->id, 'run_s:builtin:script _rmtree '.$this->encode($cmd)))
            Yii::app()->end(Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'builtin_rmtree')));
        $this->status(Yii::t('mc', 'Delete started, check console for progress'));
        Yii::app()->end('SUCCESS');
    }

    public function actionDelete($id, $path)
    {
        if (!Yii::app()->request->isPostRequest)
            Yii::app()->user->deny();

        $files = @$_POST['files'];
        $dirs = @$_POST['dirs'];
        if (!is_array($files)) $files = array();
        if (!is_array($dirs)) $dirs = array();

        if (!@Yii::app()->params['ftp_client_web_delete'])
            return $this->daemonDelete($id, $path, $files, $dirs);

        list($server, $ftp) = $this->loginFtp($id);

        $fails = array();
        $delList = array();
        $this->statusProgressInit(Yii::t('mc', 'Scanning directories...'), count($dirs));
        foreach ($dirs as $dir)
        {
            $list = $this->listFiles($ftp, $this->path($path, $dir), true);
            $this->statusProgress();
            if ($list === false)
            {
                $fails[] = $dir;
                continue;
            }
            $delList[$this->path($path, $dir)] = array('type'=>'directory', 'name'=>$dir);
            $delList = array_merge($delList, $list);
        }
        $delList = array_reverse($delList);
        foreach ($files as $file)
            $delList[$this->path($path, $file)] = array('type'=>'file', 'name'=>$file);

        $this->statusProgressInit(Yii::t('mc', 'Deleting...'), count($delList));
        foreach ($delList as $item=>$info)
        {
            if ($info['type'] == 'directory')
                $ok = ftp_rmdir($ftp, $item);
            else
                $ok = ftp_delete($ftp, $item);
            if (!$ok)
                $fails[] = $item;
            $this->statusProgress();
        }
        $this->status(Yii::t('mc', 'Done'));

        $msg = 'SUCCESS';
        if (count($fails))
        {
            $msg = Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'ftp_rmdir/ftp_delete')).':';
            $msg .= "\n".join("\n", $fails);
        }

        Yii::app()->end($msg);
    }

    public function daemonCopy($id, $path, $target, $files, $dirs)
    {
        $server = Server::model()->findByPk((int)$id);
        if (!$server)
            throw new CHttpException(404, Yii::t('mc', 'The requested page does not exist.'));

        $items = array_merge($files, $dirs);

        $cmd = array();
        foreach ($items as $item)
            $cmd[] = $this->relative($this->path($path, $item));
        $cmd[] = $this->relative($this->path($path, $target));
        if (!McBridge::get()->serverCmd($server->id, 'run_s:builtin:script _copytree '.$this->encode($cmd)))
            Yii::app()->end(Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'builtin_rmtree')));
        $this->status(Yii::t('mc', 'Copy started, check console for progress'));
        Yii::app()->end('SUCCESS');
    }

    public function actionCopy($id, $path)
    {
        if(!Yii::app()->request->isPostRequest)
            Yii::app()->user->deny();

        $target = $_POST['target'];
        $files = @$_POST['files'];
        $dirs = @$_POST['dirs'];
        if (!is_array($files)) $files = array();
        if (!is_array($dirs)) $dirs = array();

        if (!@Yii::app()->params['ftp_client_web_copy'])
            return $this->daemonCopy($id, $path, $target, $files, $dirs);

        list($server, $ftp) = $this->loginFtp($id);

        $fails = array();
        $createDirs = array();
        $this->statusProgressInit(Yii::t('mc', 'Scanning directories...'), count($dirs));
        foreach ($dirs as $dir)
        {
            $list = $this->listFiles($ftp, $this->path($path, $dir), true);
            $this->statusProgress();
            if ($list === false)
            {
                $fails[] = $dir;
                continue;
            }
            $createDirs[$this->path($path, $dir)] = array('type'=>'directory', 'name'=>$dir);
            foreach ($list as $item=>$info)
            {
                if ($info['type'] == 'directory')
                    $createDirs[$item] = $info;
                else
                    $copyFiles[$item] = $info;
            }
        }
        foreach ($files as $file)
            $copyFiles[$this->path($path, $file)] = array('type'=>'file', 'name'=>$file);

        $pathLen = ($path == '/' ? 1 : strlen($path) + 1);
        $this->statusProgressInit(Yii::t('mc', 'Copying directory structure...'), count($createDirs));
        foreach ($createDirs as $dir=>$info)
        {
            @ftp_mkdir($ftp, $this->path($target, substr($dir, $pathLen)));
            $this->statusProgress();
        }
        $fp = fopen('php://temp', 'r+');
        $this->statusProgressInit(Yii::t('mc', 'Copying files...'), count($copyFiles));
        foreach ($copyFiles as $file=>$info)
        {
            ftruncate($fp, 0);
            rewind($fp);
            if (ftp_fget($ftp, $fp, $file)
                && rewind($fp)
                && ftp_fput($ftp, $this->path($target, substr($file, $pathLen)), $fp))
            {}
            else
                $fails[] = $file;
            $this->statusProgress();
        }
        fclose($fp);

        $this->status(Yii::t('mc', 'Done'));

        $msg = 'SUCCESS';
        if (count($fails))
        {
            $msg = Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'ftp_fget/ftp_fput')).':';
            $msg .= "\n".join("\n", $fails);
        }

        Yii::app()->end($msg);
    }

    public function actionRename($id, $path)
    {
        if (!Yii::app()->request->isPostRequest)
            Yii::app()->user->deny();

        list($server, $ftp) = $this->loginFtp($id);

        $item = $_POST['item'];
        $newName = $_POST['newName'];
        if (!ftp_rename($ftp, $this->path($path, $item), $this->path($path, $newName)))
            Yii::app()->end(Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'ftp_rename')));
    }

    public function actionZip($id, $path)
    {
        if(!Yii::app()->request->isPostRequest)
            Yii::app()->user->deny();

        $server = Server::model()->findByPk((int)$id);
        if (!$server)
            throw new CHttpException(404, Yii::t('mc', 'The requested page does not exist.'));

        $zip = $_POST['zip'];
        $items = $_POST['items'];

        $path = $this->relative($path);
        $cmd = array($path ? $path : '.', $this->relative($zip));
        foreach ($items as $item)
            $cmd[] = $this->relative($item);
        if (!McBridge::get()->serverCmd($server->id, 'run_s:builtin:script _zip_cwd '.$this->encode($cmd)))
            Yii::app()->end(Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'builtin_zip_cwd')));
        $this->status(Yii::t('mc', 'Zip started, check console for progress'));
        Yii::app()->end('SUCCESS');
    }

    public function actionUnzip($id, $path)
    {
        if(!Yii::app()->request->isPostRequest)
            Yii::app()->user->deny();

        $server = Server::model()->findByPk((int)$id);
        if (!$server)
            throw new CHttpException(404, Yii::t('mc', 'The requested page does not exist.'));

        $target = $_POST['target'];
        $zips = $_POST['zips'];

        $fails = array();
        $this->statusProgressInit(Yii::t('mc', 'Sending unzip commands...'), count($zips));
        foreach ($zips as $zip)
        {
            $cmd = array(
                $this->relative($this->path($path, $zip)),
                $this->relative($target),
            );
            if (!McBridge::get()->serverCmd($server->id, 'run_s:builtin:script _unzip '.$this->encode($cmd)))
                $fails[] = $zip;
            $this->statusProgress();
        }
        $this->status(Yii::t('mc', 'Unzip started, check console for progress'));

        $msg = 'SUCCESS';
        if (count($fails))
        {
            $msg = Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'builtin_unzip')).':';
            $msg .= "\n".join("\n", $fails);
        }

        Yii::app()->end($msg);
    }

    public function getContent($ftp, $file)
    {
        $fp = fopen('php://temp', 'r+');
        if (!ftp_fget($ftp, $fp, $file, FTP_BINARY))
            return false;
        rewind($fp);
        $content = stream_get_contents($fp);
        fclose($fp);
        return $content;
    }

    public function setContent($ftp, $file, $content)
    {
        $fp = fopen('php://temp', 'r+');
        fwrite($fp, $content);
        rewind($fp);
        $success = ftp_fput($ftp, $file, $fp, FTP_BINARY);
        fclose($fp);
        return $success;
    }

    public function actionView($id, $path, $file)
    {
        $server = Server::model()->findByPk((int)$id);
        if (!$server)
            throw new CHttpException(404, Yii::t('mc', 'The requested page does not exist.'));

        list($server, $ftp) = $this->loginFtp($id, true);

        $error = '';
        $content = false;
        $size = ftp_size($ftp, $this->path($path, $file));
        if ($size === false) {}
        else if (!$this->canEdit($size))
            $error = Yii::t('mc', 'File "{name}" is too big', array('{name}'=>$file));
        else
            $content = $this->getContent($ftp, $this->path($path, $file));
        if (!$error && $content === false)
            $error = Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'getContent'));

        $this->render('view', array(
            'server'=>$server,
            'path'=>$path,
            'file'=>$file,
            'content'=>$content,
            'error'=>$error,
        ));
    }

    public function actionEdit($id, $path, $file)
    {
        $server = Server::model()->findByPk((int)$id);
        if (!$server)
            throw new CHttpException(404, Yii::t('mc', 'The requested page does not exist.'));

        list($server, $ftp) = $this->loginFtp($id, true);

        $error = '';
        $content = false;
        $readOnly = false;
        $binary = false;
        if (Yii::app()->request->isPostRequest)
        {
            $content = $_POST['content'];
            if (!$this->canEdit(strlen((string)$content)))
                $error = Yii::t('mc', 'File "{name}" is too big', array('{name}'=>$file));
            else if ($this->setContent($ftp, $this->path($path, $file), $content))
                return $this->redirect(array('ftpClient/files', 'id'=>$server->id, 'path'=>$path));
            if (!$error)
                $error = Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'setContent'));
        }
        else
        {
            $size = ftp_size($ftp, $this->path($path, $file));
            if ($size === false) {}
            else if (!$this->canEdit($size))
                $error = Yii::t('mc', 'File "{name}" is too big', array('{name}'=>$file));
            else
                $content = $this->getContent($ftp, $this->path($path, $file));
            if (!$error && $content === false)
                $error = Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'ftp_size/getContent'));
            if ($error)
                $readOnly = true;
        }

        if (!$error)
        {
            $len = strlen((string)$content);
            $content = CHtml::encode($content);
            if ($len && !strlen($content))
            {
                $error = Yii::t('mc', 'Cannot view or edit binary files');
                $readOnly = true;
            }
        }

        $this->render('edit', array(
            'server'=>$server,
            'path'=>$path,
            'file'=>$file,
            'content'=>$content,
            'error'=>$error,
            'readOnly'=>$readOnly,
        ));
    }

    public function loginFtp($id, $redirect = false)
    {
        if ($id && is_object($id))
            $server = $id;
        else
        {
            $server = Server::model()->findByPk((int)$id);
            if (!$server)
                throw new CHttpException(404, Yii::t('mc', 'The requested page does not exist.'));
        }

        $ftpSv = $this->getFtpServer($server);
        if (!isset($_SESSION['ftp_pw']))
        {
            Yii::log('No valid FTP session found, redirecting to login form');
            if ($redirect)
                return $this->redirect(array('ftpClient/login', 'id'=>$server->id));
            else
                throw new CHttpException(401, Yii::t('mc', 'FTP login expired, reload the page to login.'));
        }
        
        $func = (!!Yii::app()->params['ftp_client_ssl'] && function_exists('ftp_ssl_connect'))? 'ftp_ssl_connect' : 'ftp_connect';
        if (!function_exists($func))
            throw new CHttpException(503, Yii::t('mc', 'FTP functions are not available on this server. Please enable the PHP FTP extension or switch to net2ftp under Settings->Panel Configuration.'));
        $ftp = $func($ftpSv['ip'], $ftpSv['port'], @Yii::app()->params['timeout'] ? Yii::app()->params['timeout'] : 10);
        if (!$ftp)
            throw new CHttpException(503, Yii::t('mc', 'Failed to connect to FTP server'));
        if (!@ftp_login($ftp, $this->getUsername($server), $_SESSION['ftp_pw']))
            throw new CHttpException(403, Yii::t('mc', 'FTP login failed. Possible causes: Wrong password, no FTP account, server suspended.'));
        if (!!Yii::app()->params['ftp_client_passive'] && !@ftp_pasv($ftp, true))
            $error = Yii::t('mc', 'Failed to run FTP command ({command})', array('{command}'=>'ftp_pasv'));
        return array($server, $ftp);
    }

    function bytesFromString($val)
    {
        $val = trim((string)$val);
        $last = strtolower($val[strlen($val)-1]);
        $val = floatval($val);
        switch($last)
        {
            case 't': $val *= 1024;
            case 'g': $val *= 1024;
            case 'm': $val *= 1024;
            case 'k': $val *= 1024;
	    }
    	return $val;
    }

    function formatBytes($size)
    {
        $units = array(' B', ' KB', ' MB', ' GB', ' TB');
        for ($i = 0; $size >= 1024 && $i < 4; $i++) $size /= 1024;
        return round($size, 2).$units[$i];
    }

    function fileNavPath($server, $path, $file)
    {
        $html =  CHtml::link('<b>/&nbsp;</b>', array('files', 'id'=>$server->id, 'path'=>'/'));
        $partial = '';
        $first = true;
        foreach (explode('/', $path) as $dir)
        {
            if (!$dir)
                continue;
            $partial .= '/'.$dir;
            $html .= ($first ? '' : ' / ').CHtml::link(CHtml::encode($dir), array('files', 'id'=>$server->id, 'path'=>$partial));
            $first = false;
        }
        return $html.' / '.CHtml::encode($file);
    }

    function getIcon($file)
    {
        if ($file['type'] == 'directory')
            return 'folder';

        $dot = strrpos($file['name'], '.');
        if ($dot === false)
            $ext = '';
        else
            $ext = substr($file['name'], $dot + 1);

        $iconMap = array(
            'code' => array('asp', 'php', 'phps', 'php5', 'php7', 'html', 'htm', 'css', 'less', 'js', 'cpp', 'c', 'h', 'py', 'java', 'json', 'yml'),
            'file-alt' => array('txt', 'doc', 'docx', 'xls', 'xlsx', 'dot', 'dotx', 'odt', 'ods'),
            'image' => array('bmp', 'gif', 'jpg', 'jpeg', 'png', 'tif', 'tiff', 'raw', 'xcf', 'psd'),
            'pdf' => array('pdf'),
            'archive' => array('arc', 'tar', 'tgz', 'gz', 'bz2', 'rpm', 'deb', 'tar', 'zip', 'rar', 'xz'),
            'video' => array('mov', 'mpg', 'mpeg', 'mp4', 'mkv', 'avi', 'flv'),
            'audio' => array('wav', 'mp3', 'aac', 'flac', 'ogg'),
        );
        $icon = '';
        foreach ($iconMap as $k => $v)
        {
            if (in_array($ext, $v))
            {
                $icon = $k;
                break;
            }
        }
        return $icon ? $icon : 'file';
    }

    public function actionBrowse($id, $partial = false)
    {
        $server = Server::model()->findByPk((int)$id);
        if (!$server)
            throw new CHttpException(404, Yii::t('mc', 'The requested page does not exist.'));

        if (@$_POST['ajax'] === 'refresh')
        {
            header('Content-type: application/json');
            echo CJSON::encode($this->ajaxRefresh($server, array('log')));
            Yii::app()->end();
        }

        if (!@Yii::app()->params['ftp_client_web_zip'])
            $this->injectZip($server);

        $this->net2FtpDefines();
        global $net2ftp_result, $net2ftp_settings, $net2ftp_globals;
        require_once(dirname(__FILE__).'/../extensions/net2ftp/includes/main.inc.php');
        require_once(dirname(__FILE__).'/../extensions/net2ftp/includes/errorhandling.inc.php');

        $ftpSv = $this->getFtpServer($server);
        $sessKey = 'net2ftp_password_encrypted_'.$ftpSv['ip'].$this->getUsername($server);
        if (!isset($_SESSION[$sessKey]))
        {
            if (!isset($_SESSION['net2ftp_password_encrypted']))
            {
                Yii::log('No valid FTP session found, redirecting to login form');
                $this->redirect(array('ftpClient/login', 'id'=>$server->id));
            }
            $_SESSION[$sessKey] = $_SESSION['net2ftp_password_encrypted'];
        }
        
        set_error_handler('net2ftpErrorHandler');

        if (!@$_REQUEST['state'])
        {
            $net2ftp_globals['state'] = 'browse';
            $net2ftp_globals['state2'] = 'main';
        }
        else
        {
            $net2ftp_globals['state'] = $_REQUEST['state'];
            $net2ftp_globals['state2'] = $_REQUEST['state2'];
        }

        $net2ftp_globals['ftpserver'] = $ftpSv['ip'];
        $net2ftp_globals['ftpserverport'] = $ftpSv['port'];
        $net2ftp_globals['language'] = Yii::app()->language;
        $net2ftp_globals['username'] = $this->getUsername($server);
        $net2ftp_globals['sslconnect'] = !!Yii::app()->params['ftp_client_ssl'];
        $net2ftp_globals['passivemode'] = Yii::app()->params['ftp_client_passive'] ? 'yes' : 'no';
        $net2ftp_globals['action_url'] = CHtml::normalizeUrl(array('browse', 'id'=>$server->id, 'partial'=>$partial));
        net2ftp("sendHttpHeaders");
        //print_r($net2ftp_globals);

        if ($net2ftp_result["success"] == false)
            throw new CHttpException(404, Yii::t('mc', 'Error in the FTP client module.'));

        ob_start();
        net2ftp("printJavascript");
        $js = ob_get_contents();
        ob_clean();
        net2ftp("printCss");
        $css = ob_get_contents();
        ob_clean();
        net2ftp("printBodyOnload");
        $onload = ob_get_contents();
        ob_clean();
        global $controller;
        $controller = $this;
        net2ftp("printBody");
        $body = ob_get_contents();
        ob_clean();

        if ($net2ftp_result["success"] == false)
        {
            require_once($net2ftp_globals["application_rootdir"]."/skins/"
                .$net2ftp_globals["skin"]."/error.template.php");
            $body = ob_get_contents();
            ob_clean();
        }
        
        $func = $partial ? 'renderPartial' : 'render';
        $this->$func('browse',array(
            'js'=>$js,
            'css'=>$css,
            'onload'=>$onload,
            'body'=>$body,
            'server'=>$server,
        ));
    }

    public function encode($arr)
    {
        $out = '';
        foreach ($arr as $elem)
        {
            if (preg_match('/[" ]/', $elem))
                $elem = '"'.str_replace('"', '""', $elem).'"';
            $out .= ($out ? ' ' : '').$elem;
        }
        return $out;
    }

    public function relative($path)
    {
        return preg_replace('/^(\\/|\\.\\/)+/', '', (string)$path);
    }

    public function injectZip($server)
    {
        if (isset($_POST) && @$_POST['screen'] == 2 && Yii::app()->user->model->getServerFtpAccess($server->id) === 'rw')
        {
            $browse = true;
            if ($_POST['state'] == 'zip')
            {
                $dir = $this->relative(@$_POST['directory']);
                $dir = $dir ? $dir.'/' : '';
                $zip = $this->relative(@$_POST['zipactions']['save_filename']);
                if ($zip)
                {
                    $zip = $dir.$zip;
                    $cmd = array($zip);
                    foreach ($_POST['list'] as $entry)
                    {
                        if (!@$entry['dirfilename'])
                            continue;
                        $file = $this->relative($entry['dirfilename']);
                        if (!$file)
                            $file = '.';
                        $cmd[] = $dir.$file;
                    }
                    $ret = McBridge::get()->serverCmd($server->id, 'run_s:builtin:script _zip '.$this->encode($cmd));
                }
            }
            else if ($_POST['state'] == 'unzip')
            {
                $dir = $this->relative(@$_POST['directory']);
                $dir = $dir ? $dir.'/' : '';
                foreach ($_POST['list'] as $entry)
                {
                    $cmd = array();
                    $zip = $this->relative(@$entry['dirfilename']);
                    if (!$zip)
                        continue;
                    $target = $this->relative(@$entry['targetdirectory']);
                    if (!$target)
                        $target = '.';
                    $cmd[] = $dir.$zip;
                    $cmd[] = $target;
                    $ret = McBridge::get()->serverCmd($server->id, 'run_s:builtin:script _unzip '.$this->encode($cmd));
                }
            }
            else
                $browse = false;
            if ($browse)
            {
                $this->menu = array(
                    array('label'=>Yii::t('mc', 'Back'), 'url'=>array('ftpClient/browse', 'directory'=>@$_REQUEST['directory'],
                        'id'=>@$_GET['id']), 'icon'=>'back'),
                );
                $this->render('log',array(
                    'js'=>'',
                    'css'=>'',
                    'onload'=>'',
                    'body'=>Yii::t('mc', 'Command sent, check the server log for status'),
                    'server'=>$server,
                    'data'=>$this->ajaxRefresh($server, array('log')),
                ));
                Yii::app()->end();
            }
        }
    }
    private function ajaxRefresh($server, $type)
    {
        $all = ($type === 'all');
        if (!is_array($type))
            $type = array($type);
        $ret = array();

        $status = False;
        $seq = 0;

        if ($all || in_array('log', $type))
        {
            $error = false;
            ob_start();
            if (!Yii::app()->user->can($server->id, 'get_log'))
                $error = Yii::t('mc', 'Permission denied.');
            else if (!McBridge::get()->serverCmd($server->id, 'get_log '.((int)@$_POST['log_seq']), $log))
            {
                $error = McBridge::get()->lastError();
                if (strpos($error, 'Unknown command:') === 0)
                {
                    $error = false;
                    if (!McBridge::get()->serverCmd($server->id, 'get log', $log))
                        $error = McBridge::get()->lastError();
                }
            }

            if (strlen($error))
                echo Yii::t('mc', 'Couldn\'t get log: ').$error;
            else
            {
                for ($i = count($log) - 1; $i >= 0; $i--)
                {
                    $re = '/\\[Script .*zip\\]\s*(\\[.*zip\\])? /';
                    if (!preg_match($re, (string)@$log[$i]['line']."\n"))
                        continue;
                    echo preg_replace($re, '', (string)@$log[$i]['line']."\n");
                    if (!$seq)
                        $seq = @$log[$i]['s'];
                }
            }
            $ret['log'] = ob_get_clean();
            $ret['log_seq'] = $seq;
        }
        return $ret;
    }
}
