<?php
/**
 *
 *   Copyright © 2010-2023 by xhost.ch GmbH
 *
 *   All rights reserved.
 *
 **/

class TebexForm extends CFormModel
{
    public $use_panel_email = 1;
    public $email;
    public $have_secret = 0;
    public $secret;
    public $game_id;

    public function rules()
    {
        return array(
            array('email, secret', 'safe'),
            array('game_id', 'numerical', 'integerOnly'=>true),
            array('email', 'required', 'on'=>'checkEmail'),
            array('email', 'email', 'on'=>'checkEmail'),
            array('email', 'checkValidEmail'),
            array('use_panel_email, have_secret', 'boolean'),
            array('secret', 'required', 'on'=>'checkSecret'),
            array('secret', 'type', 'on'=>'checkSecret'),
        );
    }

    public function beforeValidate()
    {
        if (!$this->use_panel_email)
            $this->scenario = 'checkEmail';
        if ($this->have_secret)
            $this->scenario = 'checkSecret';
        return parent::beforeValidate();
    }

    public function checkValidEmail($attribute, $params)
    {
        if ($this->use_panel_email)
            $email = Yii::app()->user->model->email;
        else
            $email = $this->email;
        if (preg_match('/(example\.com|local(host)?)$/', $email))
        {
            $this->addError($attribute, $this->use_panel_email ? Yii::t('mc', 'Please set email under Users->My Profile or specify here.')
                : Yii::t('mc', 'Invalid email address.'));
            $this->use_panel_email = 0;
        }
    }

    public function attributeLabels()
    {
        return array(
            'use_panel_email'=>Yii::t('mc', 'Use Multicraft account email address'),
            'email'=>Yii::t('mc', 'Email'),
            'have_secret'=>Yii::t('mc', 'I already have a Tebex webstore'),
            'secret'=>Yii::t('mc', 'Server Key'),
            'game_id'=>Yii::t('mc', 'Game'),
        );
    }
}
