<?php
/**
 *
 *   Copyright © 2010-2023 by xhost.ch GmbH
 *
 *   All rights reserved.
 *
 **/

$this->pageTitle = Yii::app()->name.' - '.Yii::t('mc', 'Monetization');

$this->breadcrumbs=array(
    Yii::t('mc', 'Servers')=>array('index'),
    $server->name=>array('view', 'id'=>$server->id),
    Yii::t('mc', 'Monetization'),
);

$this->menu = array(
    array(
        'label'=>Yii::t('mc', 'Reset'),
        'url'=>array('server/tebex', 'id'=>$server->id),
        'linkOptions'=>array(
            'submit'=>array('server/tebex', 'id'=>$server->id),
            'confirm'=>Yii::t('mc', 'Are you sure you want to reset this page? This will delete the server secret if it exists.'),
            'csrf'=>true,
            'params'=>array('action'=>'reset')
        ),
        'visible'=>@$info['referral_id'] || @$info['secret'],
        'icon'=>'delete',
    ),
    array(
        'label'=>Yii::t('mc', 'Back'),
        'url'=>array('server/view', 'id'=>$server->id),
        'icon'=>'back',
    ),
);

Yii::app()->getClientScript()->registerCss('tebex', '
th:first-child {
    width: 150px;
    padding-left: 50px
}
');
?>

<?php if (Yii::app()->user->hasFlash('tebex')): ?>
<div class="flash-success">
    <?php echo Yii::app()->user->getFlash('tebex'); ?>
</div>
<?php endif ?>

<?php if (@$status['error']): ?>
<div class="flash-error">
    <?php echo CHtml::encode($status['error']) ?>
</div>

<?php elseif (@$info['secret']): ?>

    <?php if ($plugins): ?>
<h2>Tebex Plugin Installer</h2>
<p><?php echo Yii::t('mc', 'You can either set up the plugin manually as instructed in your Tebex panel or you can select one of the available plugins below for automatic installation.'); ?></p>
<?php
        echo CHtml::beginForm(array('server/tebex', 'id'=>$server->id));
        echo CHtml::hiddenField('action', 'install');
        $this->widget('zii.widgets.CDetailView', array(
            'data'=>$model,
            'attributes'=>array(
                array('label'=>Yii::t('mc', 'Server Type'), 'type'=>'raw', 'value'=>CHtml::dropDownList('plugin', '', $plugins)),
                array('label'=>'', 'type'=>'raw', 'value'=>CHtml::submitButton(Yii::t('mc', 'Install'))),
            )
        ));
        echo CHtml::endForm();
        ?>

    <?php else: ?>
        <div class="infoBox">
            <?php echo Yii::t('mc', 'You can set up the plugin manually as instructed in your Tebex panel.') ?>
        </div>
    <?php endif ?>

<?php elseif (@$info['referral_id']): ?>
<div class="infoBox">
    <?php
    echo Yii::t('mc', 'Your webstore setup pending. Please check back later to automatically install the server plugin from this page.');
    ?>
</div>
<?php else: ?>
<h2>Monetize Your Server with Tebex</h2>
<br/>

<?php
$form = $this->beginWidget('CActiveForm', array(
    'id'=>'tebex-form',
    'enableAjaxValidation'=>false,
    'htmlOptions'=>array('class'=>'form-horizontal'),
));

$attribs = array();

$attribs[] = array('label'=>Theme::img('icons/tebex.png', 'Tebex', array('style'=>'position:absolute')), 'type'=>'raw', 'value'=>'<b>'.Yii::t('mc', 'What is Tebex?').'</b><br/>'.Yii::t('mc', 'Tebex is a game monetization engine for game servers, offering a powerful webstore builder and control panel full of amazing features to help you sell digital products on your server.'));
$attribs[] = array('label'=>'', 'type'=>'raw', 'value'=>'<b>'.Yii::t('mc', 'How does Tebex work?').'</b><br/>'.Yii::t('mc', 'Tebex allows you to sell items, rank, pets, gems, coins, memberships, Discord ranks, or anything you\'d like connected to your server. Your store communicates with your server and releases the products once purchased via your webstore. Be sure to follow the respective game guidelines in the EULA.'));
$attribs[] = array('label'=>'', 'type'=>'raw', 'value'=>'<b>'.Yii::t('mc', 'How do I setup my Tebex Store?').'</b><br/>'.Yii::t('mc', 'Select the game type and email address to use, then click "Continue". You will receive an email with additional verification and instructions, and your store will be set up within minutes!<br/><br/>If you already have a Tebex webstore, you can enter your Server Key below to use the plugin installer.<br/><br/>Please note this is an affiliate link, and either Multicraft or the owner of this panel may receive compensation at no additional cost if you purchase a service from Tebex.'));
$attribs[] = array('label'=>Yii::t('mc', 'Game'), 'type'=>'raw',
    'value'=>$form->dropDownList($model, 'game_id', CHtml::listData($status['games'], 'id', 'name'))
        .' '.$form->error($model,'game_id'));
$attribs[] = array('label'=>'', 'type'=>'raw',
    'value'=>$form->checkBox($model,'have_secret')
        .' '.$form->labelEx($model,'have_secret',array('class'=>'form-check-label'))
        .' '.$form->error($model,'have_secret'));
$attribs[] = array('label'=>$form->labelEx($model,'secret',array('class'=>'form-check-label')), 'type'=>'raw',
    'value'=>$form->textField($model,'secret').' '.$form->error($model,'secret'), 'cssClass'=>'secret-field d-none');
$attribs[] = array('label'=>'', 'type'=>'raw',
    'value'=>$form->checkBox($model,'use_panel_email')
        .' '.$form->labelEx($model,'use_panel_email',array('class'=>'form-check-label'))
        .' '.$form->error($model,'use_panel_email'), 'cssClass'=>'email-title');
$attribs[] = array('label'=>$form->labelEx($model,'email',array('class'=>'form-check-label')), 'type'=>'raw',
    'value'=>$form->textField($model,'email').' '.$form->error($model,'email'), 'cssClass'=>'email-field d-none');
$attribs[] = array('label'=>'', 'type'=>'raw', 'value'=>CHtml::submitButton(Yii::t('mc', 'Continue')));

echo CHtml::script('
    $(document).ready(function () {
        $(".email-field").removeClass("d-none");
        $(".secret-field").removeClass("d-none");
        
        $("#TebexForm_use_panel_email").on("change", function () {
            $(".email-field").toggle(!$(this).is(":checked"));
        });
        $("#TebexForm_have_secret").on("change", function () {
            $(".secret-field").toggle($(this).is(":checked"));
            $(".email-title").toggle(!$(this).is(":checked"));
            $(".email-field").toggle(!$(this).is(":checked") && !$("#TebexForm_use_panel_email").is(":checked"));
        });
        $("#TebexForm_use_panel_email").trigger("change");
        $("#TebexForm_have_secret").trigger("change");
    });
');

$this->widget('zii.widgets.CDetailView', array(
    'data'=>$model,
    'attributes'=>$attribs,
));

$this->endWidget();
?>
<?php endif; ?>
