#!/bin/bash

set -euo pipefail

command -v curl &>/dev/null || { echo "The modpack installer needs the 'curl' command but it is not installed"; exit 1; }
command -v python3 &>/dev/null || { echo "The modpack installer needs Python 3 but it is not installed"; exit 1; }

mkdir -p "${SERVER_DIR}/.multicraft_modpack"
cd "${SERVER_DIR}/.multicraft_modpack"

echo "Downloading latest Modpack Installer"
curl -sSLG "https://plugins.multicraft.org/api/v2/tools/modpack_installer" --data-urlencode "data@params" -o modpack_installer
curl -sSL  "https://plugins.multicraft.org/api/v2/tools/modpack_installer_checksum" -o modpack_installer_checksum

expected_checksum=$(cat modpack_installer_checksum)
actual_checksum=$(sha256sum modpack_installer | awk '{ print $1 }')

if [ "$expected_checksum" != "$actual_checksum" ]; then
    echo "Failed to verify modpack installer checksum. The file may be corrupted." >&2
    exit 1
fi

echo "Running Modpack Installer"
chmod 755 modpack_installer
exec ./modpack_installer
